'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _lodash = require('lodash');

var _View = require('./View');

var _View2 = _interopRequireDefault(_View);

var _ContentContainer = require('../ContentContainer');

var _ContentContainer2 = _interopRequireDefault(_ContentContainer);

var _EditIndicator = require('../EditIndicator');

var _EditIndicator2 = _interopRequireDefault(_EditIndicator);

var _createThumbnail = require('./createThumbnail');

var _createThumbnail2 = _interopRequireDefault(_createThumbnail);

var _fileToImage = require('./fileToImage');

var _fileToImage2 = _interopRequireDefault(_fileToImage);

var _createImageVariations = require('./createImageVariations');

var _createImageVariations2 = _interopRequireDefault(_createImageVariations);

var _content = require('../../actions/content');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var supportedMimes = ['image/jpeg', 'image/png', 'image/gif'];

function isFileSupported(file) {
  return supportedMimes.indexOf(file.type) > -1;
}

var styles = {
  container: {
    position: 'relative',
    minHeight: 50,
    minWidth: 50
  },

  dragOver: {
    background: 'yellow'
  },

  removeButton: {
    position: 'absolute',
    top: 0,
    right: 0
  },

  instructions: {
    position: 'absolute',
    top: '50%',
    left: '50%',
    transform: 'translate(-50%, -50%)'
  }
};

var EditImage = function (_React$Component) {
  _inherits(EditImage, _React$Component);

  function EditImage() {
    var _ref;

    var _temp, _this, _ret;

    _classCallCheck(this, EditImage);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = EditImage.__proto__ || Object.getPrototypeOf(EditImage)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      dragOver: false
    }, _this.onDragOver = function (e) {
      e.stopPropagation();
      e.preventDefault();

      _this.setState({
        dragOver: true
      });
    }, _this.onDragLeave = function (e) {
      _this.setState({
        dragOver: false
      });
    }, _this.onDrop = function (e) {
      e.preventDefault();

      _this.setState({
        dragOver: false
      });

      var file = (0, _lodash.first)(e.dataTransfer.files);

      if (file && isFileSupported(file)) {
        if (file.type === 'image/gif') {
          _this.createOriginalUpload(file);
        } else {
          _this.createResizedUpload(file);
        }
      } else {
        var supportedFormatsFriendly = supportedMimes.map(function (mime) {
          return mime.split('/')[1];
        }).join(', ');
        window.alert('Only images of types ' + supportedFormatsFriendly + ' are supported');
      }
    }, _this.onRemoveImageClick = function () {
      _this.props.setValue({});
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(EditImage, [{
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {}
  }, {
    key: 'createOriginalUpload',
    value: function createOriginalUpload(file) {
      var _this2 = this;

      (0, _fileToImage2.default)(file, function (image) {
        var uploadPath = 'uploads/' + Date.now();

        var uploadAction = (0, _content.setUpload)(_this2.props.path, {
          preview: image.src,
          files: {
            path: '' + uploadPath,
            data: file
          }
        });

        var valueAction = (0, _content.set)(_this2.props.path, {
          path: '/' + uploadPath,
          type: file.type,
          width: image.width,
          height: image.height,
          thumbnail: (0, _createThumbnail2.default)(image, 10).src
        });

        _this2.context.flatpack.store.dispatch(uploadAction);
        _this2.context.flatpack.store.dispatch(valueAction);
      });
    }
  }, {
    key: 'createResizedUpload',
    value: function createResizedUpload(file) {
      var _this3 = this;

      (0, _fileToImage2.default)(file, function (image) {
        (0, _createImageVariations2.default)(file, function (variations) {
          var uploadPath = 'uploads/' + Date.now();

          var uploadAction = (0, _content.setUpload)(_this3.props.path, {
            preview: image.src,
            files: variations.map(function (_ref2) {
              var width = _ref2.width,
                  blob = _ref2.blob;
              return {
                path: uploadPath + '/' + width,
                data: blob
              };
            })
          });

          var valueAction = (0, _content.set)(_this3.props.path, {
            path: '/' + uploadPath,
            type: file.type,
            width: image.width,
            height: image.height,
            variations: variations.map(function (variation) {
              return (0, _lodash.pick)(variation, ['width', 'height']);
            }),
            thumbnail: (0, _createThumbnail2.default)(image, 10).src
          });

          _this3.context.flatpack.store.dispatch(uploadAction);
          _this3.context.flatpack.store.dispatch(valueAction);
        });
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var value = this.props.value;
      var dragOver = this.state.dragOver;


      var containerStyle = _extends({}, styles.container, dragOver && styles.dragOver);

      return _react2.default.createElement(
        _EditIndicator2.default,
        null,
        _react2.default.createElement(
          'div',
          { style: containerStyle, onDrop: this.onDrop, onDragOver: this.onDragOver, onDragLeave: this.onDragLeave },
          _react2.default.createElement(_View2.default, { path: this.props.path, style: this.props.style }),
          value.path && _react2.default.createElement(
            'button',
            { style: styles.removeButton, onClick: this.onRemoveImageClick },
            'Remove image'
          ),
          !value.path && _react2.default.createElement(
            'div',
            { style: styles.instructions },
            'Drop image file here\u2026'
          )
        )
      );
    }
  }]);

  return EditImage;
}(_react2.default.Component);

EditImage.propTypes = {
  path: _react2.default.PropTypes.string.isRequired,
  value: _react2.default.PropTypes.object,
  setValue: _react2.default.PropTypes.func.isRequired,
  style: _react2.default.PropTypes.object
};
EditImage.defaultProps = {
  value: {},
  attrs: {}
};
EditImage.contextTypes = {
  flatpack: _react2.default.PropTypes.object
};
exports.default = (0, _ContentContainer2.default)(EditImage);