'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = renderRoutes;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _history = require('history');

var _server = require('react-dom/server');

var _Provider = require('./components/Provider');

var _Provider2 = _interopRequireDefault(_Provider);

var _objectToString = require('./utils/objectToString');

var _objectToString2 = _interopRequireDefault(_objectToString);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function inject(html, content, scripts) {
  var result = html;

  var injectBefore = '</body>';
  var injection = '\n    <script>\n      window.content = \'' + (0, _objectToString2.default)(content) + '\';\n      window.scripts = \'' + (0, _objectToString2.default)(scripts) + '\';\n    </script>\n    ' + scripts.map(function (script) {
    return '\n      <script src="' + script + '"></script>\n    ';
  }) + '\n  ';

  if (result.indexOf(injectBefore) === -1) {
    result += injection;
  } else {
    result = result.replace(injectBefore, '' + injection + injectBefore);
  }

  return result;
}

function renderWithProvider(component, config, content, scripts) {
  var html = (0, _server.renderToStaticMarkup)(_react2.default.createElement(
    _Provider2.default,
    { content: content, config: config, assets: {}, scripts: scripts, history: (0, _history.createMemoryHistory)() },
    component
  ));

  return inject(html, content, scripts);
}

function renderRoutes(_ref, callback) {
  var config = _ref.config,
      content = _ref.content,
      scripts = _ref.scripts,
      version = _ref.version;

  var prefix = version ? version + '-' : '';
  var routes = config.routes(content);
  var template = config.template;
  var files = {};

  try {
    // Route files
    routes.forEach(function (route) {
      if (route.path.slice(-1) !== '/') {
        callback(new Error('Route path \'' + route.path + '\' is missing trailing slash.'));
      }

      var filepath = ('/' + route.path + '/' + prefix + 'index.html').split('/').filter(function (v) {
        return !!v;
      }).join('/');
      var html = renderWithProvider(route.component, config, content, scripts);

      files[filepath] = template(html, route.title);
    });

    // 404 file
    if (config.notFoundRoute) {
      var html = renderWithProvider(config.notFoundRoute.component, config, content, scripts);
      files[prefix + '404.html'] = template(html, config.notFoundRoute.title, content);
    }

    callback(null, files);
  } catch (err) {
    callback(err);
  }
}