'use strict';

var mimeTypes = require('mime-types');
var MemoryFS = require('memory-fs');
var webpack = require('webpack');
var getWebpackConfig = require('../../getWebpackConfig');

var icon = '🛠';

var config = getWebpackConfig('production');
var compiler = webpack(config);
var fs = new MemoryFS();

module.exports = function (req, res) {
  console.log(icon, 'Compiling production scripts…');

  compiler.outputFileSystem = fs;

  compiler.run(function (error, stats) {
    if (error) {
      console.log(error);
      return res.status(500).json({ error: error });
    }

    var assets = stats.toJson().assets;

    console.log(icon, 'Compiled', assets.length, 'scripts');

    var files = assets.map(function (asset) {
      console.log(icon, 'Reading ' + asset.name + '\u2026');

      var path = config.output.path + '/' + asset.name;

      var data = fs.readFileSync(path);

      return {
        path: asset.name,
        type: mimeTypes.lookup(asset.name),
        data: data.toString('base64')
      };
    });

    console.log(icon, 'Done!');

    return res.status(200).json({
      files: files,
      stats: stats.toJson()
    });
  });
};