'use strict';

var webpack = require('webpack');
var chalk = require('chalk');
var cloneDeep = require('lodash').cloneDeep;
var requireFromString = require('require-from-string');
var MemoryFS = require('memory-fs');
var configPath = require('../../config/path');
var configDefaults = require('../../config/defaults');
var getWebpackConfig = require('../../getWebpackConfig');
var webpackBuilder = require('./webpackBuilder');

var fs = new MemoryFS();
var webpackConfig = cloneDeep(getWebpackConfig('common'));

webpackConfig.target = 'node';
webpackConfig.output.libraryTarget = 'commonjs';

webpackConfig.entry = {
  config: configPath
};

if (!webpackConfig.resolve) {
  webpackConfig.resolve = {};
}

webpackConfig.resolve.aliasFields = ['server'];
webpackConfig.resolve.mainFields = ['module', 'main'];

var compiler = webpack(webpackConfig);

compiler.outputFileSystem = fs;

function logError(error) {
  chalk.red('✘ Build error!');
  console.log(error.message);
}

var builder = webpackBuilder(compiler, 'config');

var transformer = function transformer(req, res, next) {
  var filename = res.locals.webpack.config.stats.toJson().assetsByChunkName.config;
  var configPath = webpackConfig.output.path + '/' + filename;

  if (fs.existsSync(configPath)) {
    fs.readFile(configPath, 'utf-8', function (err, data) {
      if (err) {
        logError(err);
      }

      try {
        var getConfig = requireFromString(data).default;
        res.locals.config = configDefaults(getConfig);
        next();
      } catch (e) {
        logError(e);
      }
    });
  } else {
    logError(new Error('Could not read ' + configPath + ' from MemoryFS filesystem'));
  }
};

module.exports = [builder, transformer];