'use strict';

var path = require('path');
var chalk = require('chalk');
var webpack = require('webpack');
var MemoryFS = require('memory-fs');
var webpackBuilder = require('./webpackBuilder');
var getWebpackConfig = require('../../getWebpackConfig');
var fetchRemoteContent = require('../../fetchRemoteContent');
var renderRoutes = require('../../renderRoutes').default;

var fs = new MemoryFS();
var webpackConfig = getWebpackConfig('development');
var compiler = webpack(webpackConfig);
var builder = webpackBuilder(compiler, 'scripts');

compiler.outputFileSystem = fs;

var statics = function statics(req, res, next) {
  var assets = [];
  var publicPath = webpackConfig.output.publicPath || '';
  var assetsByChunkName = res.locals.webpack.scripts.stats.toJson().assetsByChunkName;

  Object.keys(assetsByChunkName).map(function (chunkName) {
    var chunkValue = assetsByChunkName[chunkName];

    if (chunkValue instanceof Array) {
      chunkValue.forEach(function (value) {
        return assets.push(publicPath + value);
      });
    } else {
      assets.push(publicPath + chunkValue);
    }
  });

  var assetIndex = assets.indexOf(req.path);

  if (assetIndex > -1) {
    var filePath = path.join(webpackConfig.output.path, assets[assetIndex]);

    if (fs.existsSync(filePath)) {
      fs.readFile(filePath, 'utf-8', function (err, data) {
        if (err) {
          console.log(chalk.red('\u2718 Error reading static file from memory:', filePath));
          return next();
        }

        res.status(200).send(data);
      });
    }
  } else {
    next();
  }
};

var pages = function pages(req, res, next) {
  var config = res.locals.config;
  var clientScripts = res.locals.webpack.scripts.stats.toJson().assetsByChunkName.client;
  var clientScript = clientScripts instanceof Array ? clientScripts[0] : clientScripts;
  var scripts = [path.join(webpackConfig.output.publicPath, clientScript)];

  if (config) {
    console.log('🕗 Fetching remote content…');

    fetchRemoteContent(function (err, content) {
      if (err) {
        console.log(chalk.red('\u2718 Error fetching remote config:', err.message));
      }

      console.log('🕗 Rendering routes…');

      renderRoutes({ config: config, content: content, scripts: scripts }, function (err, files) {
        if (err) {
          console.log(chalk.red('\u2718 Error rendering routes:'));
          console.error(err);
          return res.status(500).send(err.toString());
        }

        var matchedKey = Object.keys(files).find(function (key) {
          return key === req.path.slice(1) || key === path.join(req.path.slice(1), 'index.html');
        });

        var notFoundKey = '404.html';
        var notFoundFile = files[notFoundKey];

        if (matchedKey) {
          res.status(200).send(files[matchedKey]);
          console.log(chalk.green('\u2714\uFE0E Successfully served \'' + req.path + '\''));
        } else if (notFoundFile) {
          res.status(200).send(notFoundFile);
          console.log(chalk.green('\u2714\uFE0E Successfully served \'' + notFoundKey + '\''));
        } else {
          next();
        }
      });
    });
  }
};

module.exports = [builder, statics, pages];