'use strict';

var chalk = require('chalk');

module.exports = function (compiler, name) {
  var state = {
    stats: null,
    compiling: false,
    queue: []
  };

  compiler.plugin('compile', function () {
    state.compiling = true;
    console.log('\uD83D\uDD57 Bundling ' + name + '\u2026');
  });

  compiler.plugin('done', function (stats) {
    var errors = stats.compilation.errors && stats.compilation.errors.length;

    state.compiling = false;
    state.stats = stats;

    if (errors) {
      console.log(stats.toString({
        hash: false,
        version: false,
        chunks: false,
        assets: false,
        colors: true
      }));
    }

    console.log(errors ? chalk.red('\u2718 Error bundling ' + name) : chalk.green('\u2714\uFE0E Successfully bundled ' + name));

    if (state.queue.length) {
      state.queue.forEach(function (next) {
        return next();
      });
      state.queue = [];
    }
  });

  compiler.watch(null, function (err, stats) {
    if (err) {
      throw err;
    }
  });

  return function (req, res, next) {
    if (!res.locals.webpack) {
      res.locals.webpack = {};
    }

    var proceed = function proceed() {
      res.locals.webpack[name] = state;
      next();
    };

    if (state.compiling) {
      console.log('\uD83D\uDD57 Bundling in progress (' + name + '). Queueing request\u2026');
      state.queue.push(proceed);
    } else {
      res.locals.webpack[name] = state;
      proceed();
    }
  };
};